import copy
import types
import matplotlib.pyplot as plt
import random as rnd
import collections.abc

import copy
import types
import matplotlib.pyplot as plt
import random as rnd
import collections.abc

_list_access_cnt = 0

class AccessCountingList(list):
   def __init__(self, *args):
       super().__init__(*args)
   def __getitem__(self, idx):
       global _list_access_cnt
       _list_access_cnt += 1
       if isinstance(idx, slice):
         return AccessCountingList(super().__getitem__(idx))
       else:
         return super().__getitem__(idx)
   def __setitem__(self, key, value):
       global _list_access_cnt
       _list_access_cnt += 1
       return super().__setitem__(key, value)
   def __iter__(self):
       self.iterator = AccessCountIterator(super().__iter__())
       return self.iterator
   def __add__(self, arg):
     return AccessCountingList(super().__add__(arg))

class AccessCountIterator(collections.abc.Iterator):
   def __init__(self, it, *args):
       self._it = it
   def __iter__(self):
       return self
   def __next__(self):
       global _list_access_cnt
       _list_access_cnt += 1
       return self._it.__next__()

def make1d(n, value=0, random=False):
    if not type(n) is int:
        raise TypeError("make1d: first argument should be an integer")
    if random:
        return AccessCountingList(rnd.random() for i in range(0, n))
    else:
        return AccessCountingList(copy.deepcopy(value) for i in range(0, n))

def make2d(n, m, value=0, random=False):
    if not type(n) is int:
        raise TypeError("make2d: first argument should be an integer")
    if not type(m) is int:
        raise TypeError("make2d: second argument should be an integer")
    return AccessCountingList(make1d(m, value=value, random=random)
                              for i in range(0, n))

def make3d(n, m, k, value=0, random=False):
    if not type(n) is int:
        raise TypeError("make3d: first argument should be an integer")
    if not type(m) is int:
        raise TypeError("make3d: second argument should be an integer")
    if not type(k) is int:
        raise TypeError("make3d: third argument should be an integer")
    return AccessCountingList(make2d(m, k, value=value, random=random)
                              for i in range(0, n))

def print2d(data, rowLabels=None, colLabels=None): #not used in the textbook
    if not (isinstance(data, collections.abc.Sequence) and
            isinstance(data[0], collections.abc.Sequence)):
        raise TypeError("print2d: argument should be a 2d-array")
    tbl = plt.table(cellText=[[str(j) for j in i] for i in data],
                    bbox=[0,0,1,1], cellLoc='center', loc='top',
                    rowLabels=rowLabels, colLabels=colLabels)
    tbl.auto_set_font_size(False)
    tbl.set_fontsize(16)
    plt.axis('off')
    plt.show()



from matplotlib import pyplot
import time
import types
import collections

from matplotlib import pyplot
import time
import types
import collections

_list_access_cnt = 0
_recursion_cnt = 0

def argToStr(arg):
    def argToStrMain(arg):
        if isinstance(arg, collections.abc.Sequence) and len(arg) > 3:
            return "[" + argToStrMain(arg[0]) + "," + argToStrMain(arg[1]) + ", ...]"
        else:
            return str(arg)
    if isinstance(arg, tuple):
        return argToStrMain(arg)
    else:
        return "(" + argToStrMain(arg) + ")"

def evalWithTime(fn, f, arg, criteria, c):
    global _list_access_cnt
    global _recursion_cnt
    _list_access_cnt = 0
    _recursion_cnt = 0
    print("evaluating ", fn + argToStr(arg), "... ", end="")
    start = time.perf_counter()
    for i in range(c):
        if isinstance(arg, tuple):
            f(*arg)
        else:
            f(arg)
    et = (time.perf_counter() - start) / c
    print("finished in ", et, " seconds.")
    if criteria == "time":
        return et
    elif criteria == "access":
        return _list_access_cnt / c
    elif criteria == "recursion":
        return _recursion_cnt / c
    else:
        raise TypeError("bench: unsupported criteria")

def count_recursive_call(func):
   def wrapper(*args, **kwargs):
       global _recursion_cnt
       _recursion_cnt += 1
       return func(*args, **kwargs)
   return wrapper

def default_measure(x):
    if isinstance(x, int) or isinstance(x, float):
        return x
    if isinstance(x, tuple):
        return sum([default_measure(y) for y in x])
    if isinstance(x, collections.abc.Sized):
        return len(x)
    raise TypeError("bench: the size of the argument is unknown. Specify the mesure.")

def bench(f, args, criteria="time", measure=default_measure, count=1):
    if not hasattr(f, '__call__'):
        raise TypeError("bench: first argument should be a function")
    if not isinstance(args, collections.abc.Sequence):
        raise TypeError("bench: second argument should be a series of arguments")
    x = [measure(i) for i in args]
    func = count_recursive_call(f)
    globals()[f.__name__] = func
    y = [evalWithTime(f.__name__, f ,i, criteria, count) for i in args]
    globals()[f.__name__] = f
    return (x, y)


def plot(d, xlogscale=False, ylogscale=False):
    if xlogscale:
        pyplot.xscale("log")
    else:
        pyplot.xscale("linear")
    if ylogscale:
        pyplot.yscale("log")
    else:
        pyplot.yscale("linear")
    pyplot.plot(d[0],d[1])
    pyplot.show()




# -*- encoding: utf-8 -*-

from . import array
from . import plot
from . import bench
from . import excheck

def lifegame_glider():
    return [[0,0,0,0,0,0,0,0],
            [0,0,0,0,0,0,0,0],
            [0,0,0,0,0,0,0,0],
            [0,0,0,0,0,0,0,0],
            [0,0,0,0,1,1,1,0],
            [0,0,0,0,1,0,0,0],
            [0,0,0,0,0,1,0,0],
            [0,0,0,0,0,0,0,0]]

def lifegame_acorn():
     # Corderman's Acorn
    image = array.make2d(60,80)
    x = 60
    y = 30
    for i in [[0,1],[1,3],[2,0],[2,1],[2,4],[2,5],[2,6]]:
        image[y + i[0]][x + i[1]] = 1
    return image

import random as rnd

# 身長体重データ疑似生成用
def gen_hw_data():
    s = rnd.randint(0,1)
    h = int(rnd.gauss(155+s*18, 10))
    t = rnd.randint(0,1)
    wd = (h/100) ** 2 * (19+3*t)
    w = int(wd + rnd.betavariate(2,10) * 200 - 15)
    return (h,w)

# バネの延びデータ疑似生成用
def gen_spring_data():
    w = rnd.randint(0,200)
    l = int(2 * w * rnd.gauss(1, 0.1))
    return (w,l)

# benchでの性能計測用
_list_access_cnt = 0
_recursion_cnt = 0



